//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "record_loop.h"

RecordLoop::RecordLoop(shared_t *sharedRes, QThread *thr, QObject *parent) : QProcess(parent)
{
    checkTimer = 0;
    status     = 0;
    shared     = sharedRes;

    connect(thr, &QThread::started,  this, &RecordLoop::init);
    connect(thr, &QThread::finished, this, &RecordLoop::deleteLater);

    connect(this, &RecordLoop::readyReadStandardOutput, this, &RecordLoop::resetTime);
    connect(this, &RecordLoop::readyReadStandardError,  this, &RecordLoop::resetTime);
    connect(this, &RecordLoop::started,                 this, &RecordLoop::resetTime);
    connect(this, &RecordLoop::finished,                this, &RecordLoop::segFinished);

    moveToThread(thr);
}

RecordLoop::~RecordLoop()
{
    terminate();
    waitForFinished();
}

void RecordLoop::init()
{
    checkTimer = new QTimer(this);

    connect(checkTimer, &QTimer::timeout, this, &RecordLoop::restart);

    checkTimer->setSingleShot(true);
    checkTimer->setInterval(3000);

    setupBuffDir(shared->buffPath);
    restart();
}

void RecordLoop::segFinished(int retCode)
{
    status = retCode;

    restart();
}

QStringList RecordLoop::camCmdFromConf()
{
    auto tobj = QDateTime::currentDateTimeUtc();
    auto ret  = QStringList();

    ret.append("ffmpeg");
    ret.append("-hide_banner");
    ret.append("-y");
    ret.append("-i");
    ret.append(shared->recordUri);

    if (shared->recordUri.contains("rtsp"))
    {
        ret.append("-rtsp_transport");
        ret.append("udp");
    }

    if (shared->vidCodec != "copy")
    {
        ret.append("-vf");
        ret.append("fps=" + QString::number(shared->recFps) + ",scale=" + shared->recScale);
    }

    ret.append("-vcodec"); ret.append(shared->vidCodec);
    ret.append("-acodec"); ret.append(shared->audCodec);
    ret.append("-reset_timestamps");
    ret.append("1");
    ret.append("-sc_threshold");
    ret.append("0");
    ret.append("-g");
    ret.append("2");
    ret.append("-force_key_frames");
    ret.append("expr:gte(t, n_forced * 2)");
    ret.append("-t");
    ret.append(QString::number(SEG_SIZE));
    ret.append(shared->buffPath + "/live/" + tobj.toString(DATETIME_FMT) + shared->streamExt);

    return ret;
}

QString RecordLoop::statusLine()
{
    if ((status == 0) || (status == 255))
    {
        return "OK  ";
    }
    else
    {
        return "FAIL";
    }
}

void RecordLoop::resetTime()
{
    checkTimer->start();
}

void RecordLoop::restart()
{
    if (state() == QProcess::Running)
    {
        terminate();
    }
    else
    {
        resetTime();

        auto args = camCmdFromConf();

        setProgram(args[0]);
        setArguments(args.mid(1));

        start();
    }
}
